<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Mar 31, 2020
 */

/**
 * acronis Model.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Model_Acronis
{
    private $_instance_id;
    private $_agent_id;
    private $_RecoveryApi;


    /**
     * Converts bytes to GB and returns size with GB suffix.
     *
     * @param float $bytes
     * @return string
     */
    protected function _convertBytes($bytes)
    {
        if ($bytes === 0) {
            return '0 GB';
        }

        $units = array('B', 'KB', 'MB', 'GB', 'TB');

        $base = log($bytes) / log(1024);
        $suffix = $units[floor($base)];


        $result = pow(1024, $base - floor($base));

        $result = round($result, 1);
        return "$result $suffix";
    }

    /**
     * Get acronis plugin.
     *
     * @return Plugin_Acronis
     */
    protected function _getPlugin()
    {
        return IW::PluginManager()->getPlugin('acronis');
    }

    /**
     * Get cache.
     *
     * @return Model_Cache|Mock
     */
    protected function _getCache()
    {
        return $this->_getPlugin()->getCache();
    }

    /**
     * Configure date.
     *
     * @param string $date
     * @return array
     */
    public static function formatDate($date)
    {
        if (empty($date)) {
            return 0;
        }

        $time = strtotime($date);

        $weekday = date('l', $time);
        $day = date('d', $time);
        $month = date('F', $time);
        $year = date('Y', $time);
        $hour = date('H', $time);
        $minutes = date('i', $time);

        return array(
            'year' => $year,
            'month' => $month,
            'day' => $day,
            'weekday' => $weekday,
            'time' => "$hour:$minutes"
        );
    }

    /**
     * @param integer $timestamp
     * @return string
     */
    public static function getDateString($timestamp)
    {
        return date('Y:m:d:H:i:s', $timestamp);
    }


    /**
     * @return Model_Cache_Backup
     */
    protected function _getBackupsCache()
    {
        return new Model_Cache_Backup();
    }

    /**
     * Get resources api
     *
     * @return Model_Api_Public_Resources
     */
    public function getResourcesApi(): Model_Api_Public_Resources
    {
        return new Model_Api_Public_Resources();
    }

    /**
     * get applications api
     *
     * @return Model_Api_Public_Applications
     */
    public function getApplicationsApi(): Model_Api_Public_Applications
    {
        return new Model_Api_Public_Applications();
    }

    /**
     * Get policies api
     *
     * @return Model_Api_Public_Policies
     */
    public function getPoliciesApi(): Model_Api_Public_Policies
    {
        return new Model_Api_Public_Policies();
    }

    /**
     * Get alerts api
     */
    public function getAlertsApi(): Model_Api_Public_Alerts
    {
        return new Model_Api_Public_Alerts();
    }

    /**
     * Get backups api
     */
    public function getBackupsApi(): Model_Api_Private_VaultManager_Backups
    {
        return new Model_Api_Private_VaultManager_Backups();
    }

    /**
     * Get activities api
     */
    public function getActivitiesApi(): Model_Api_Private_Ams_Activities
    {
        return new Model_Api_Private_Ams_Activities();
    }

    /**
     * Get tasks api
     */
    public function getTasksApi(): Model_Api_Public_TaskManager_Tasks
    {
        return new Model_Api_Public_TaskManager_Tasks();
    }

    /**
     * get archives api
     */
    public function getArchivesApi(): Model_Api_Private_Ams_Archives
    {
        return new Model_Api_Private_Ams_Archives();
    }

    /**
     * get subscriptions api
     */
    public function getSubscriptionsApi(): Model_Api_Private_Subscriptions
    {
        return new Model_Api_Private_Subscriptions();
    }

    /**
     * Get recovery api
     */
    public function getRecoveryApi(): Model_Api_Private_Ams_RecoveryPlan
    {
        if (!isset($this->_RecoveryApi)) {
            $this->_RecoveryApi = new Model_Api_Private_Ams_RecoveryPlan();
        }

        return $this->_RecoveryApi;
    }

    /**
     * Get IDP Api. This is the endpoint for authorization, e.g. to
     * get a one time login link.
     *
     * @return Model_Api_Public_Idp
     */
    public function getIdpApi(): Model_Api_Public_Idp
    {
        return new Model_Api_Public_Idp();
    }

    /**
     * Get instance id. This is the machine/resource id, specific to the server.
     *
     * @return string
     */
    public function getInstanceId()
    {
        if (!isset($this->_instance_id)) {
            $Cache = $this->_getCache();

            if ($Cache->instanceIdExists()) {
                $this->_instance_id = $Cache->getInstanceId();
            } else {
                $this->_setServerIds();
            }
        }

        return strtolower($this->_instance_id);
    }

    /**
     * Set ids needed to identify server & backup agent
     */
    private function _setServerIds()
    {
        $Cache = $this->_getCache();

        $ResourceMachine = $this->getResourcesApi()->getResourceByHostname();

        $this->_instance_id = $ResourceMachine->getInstanceId();
        $this->_agent_id = $ResourceMachine->getAgentId();

        $Cache->cacheAgentId($this->_agent_id);
        $Cache->cacheInstanceId($this->_instance_id);
    }

    /**
     * Get instance id. This is the machine/resource id, specific to the server.
     *
     * @return string
     */
    public function getAgentId()
    {
        if (!isset($this->_agent_id)) {
            $Cache = $this->_getCache();

            if ($Cache->isCached($Cache::CACHE_AGENT_ID)) {
                $this->_agent_id = $Cache->getAgentId();
            } else {
                $this->_setServerIds();
            }
        }

        return strtolower($this->_agent_id);
    }

    /**
     * Get acronis user id
     */
    protected function _getUserId(): string
    {
        $OAuthClient =  $this->_getPlugin()->getOAuthApiClient();
        return $OAuthClient->getUserId();
    }
}
