<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 20, 2020
 */

/**
 * Backup Plan class.
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
abstract class Model_Backups_Plan_Policy
{
    /**
     * parent policy for other policies, such as policy.backup.machine
     */
    const TYPE_POLICY_PROTECTION_TOTAL = 'policy.protection.total';

    /**
     * Policy responsible for backup actions - e.g. run backup & schedule backups
     */
    const TYPE_POLICY_BACKUP_MACHINE = 'policy.backup.machine';

    /**
     * backup.machine sometimes show up as invalid? at different endpoints. Not sure why
     */
    const TYPE_POLICY_BACKUP_INVALID = 'policy.backup.invalid';

    /**
     * plans that cover certain directories instead of entire server. Currently not supported
     */
    const TYPE_POLICY_BACKUP_FILES = 'policy.backup.files';

    const ID = 'id';
    const ENABLED = 'enabled';
    const NAME = 'name';
    const TYPE = 'type';
    const SETTINGS = 'settings';
    const PRE_POST_COMMANDS = 'pre_post_commands';
    const SCHEDULE = 'schedule';
    const SCHEDULING = 'scheduling';
    const BACKUP_SETS = 'backup_sets';

    const TYPE_PRE_COMMAND = 'pre_command';
    const TYPE_PRE_DATA_COMMAND = 'pre_data_command';
    const TYPE_POST_COMMAND = 'post_command';

    const PRE_SCRIPT_CONFIG_CMD = '/usr/lib/Acronis/BackupAndRecovery/webcpprecapture';
    const POST_SCRIPT_CONFIG_CMD =  '/usr/lib/Acronis/BackupAndRecovery/webcppostcapture';

    private $_id;
    private $_name;
    private $_type;
    private $_enabled;
    private $_parent_id;
    private $_resource_ids;
    private $_settings;

    /**
     * Constructor.
     *
     * @param array $policy
     * @param array $context
     */
    public function __construct(array $policy, array $context = null)
    {
        $this->_context = $context;
        if ($this->_context) {
            $this->_resource_ids = $context['items'];
        }

        $this->_id = $policy[self::ID];
        $this->_type = $policy[self::TYPE];
        $this->_name = $policy[self::NAME];
        $this->_enabled = $policy[self::ENABLED];
        $this->_parent_id = $policy['parent_ids'][0]; // assuming only one parent - is this ok?
        $this->_settings =  $policy[self::SETTINGS];
    }

    /**
     * @return array
     */
    protected function _getContext()
    {
        return $this->_context;
    }

    /**
     * Will only include settings if url includes query: include_settings=true
     *
     * Only type policy.backup.machine seem to include this
     * (or all none-root policies, but that is the only non-root policy we are interested in)
     *
     * @return array|null
     */
    protected function _getSettings()
    {
        return $this->_settings;
    }


    /**
     * Only applicable for non-root policies (policy.backup.machine in our case)
     *
     * @return string
     */
    public function getParentId(): string
    {
        return $this->_parent_id;
    }

    /**
     * Check if policy has parent (policy.backup.machine has parent policy.protection.total)
     */
    public function hasParent()
    {
        return isset($this->_parent_id);
    }


    /**
     * Get plan details as array.
     *
     * @return array
     */
    public function getPlanDetails()
    {
        return array(
            self::ENABLED => $this->_enabled,
            self::ID => $this->_id,
            self::NAME => $this->_name,
            self::TYPE => $this->_type,
        );
    }

    /**
     * Check if plan is enabled.
     *
     * @return boolean
     */
    public function isEnabled()
    {
        return $this->_enabled;
    }

    /**
     * Get type.
     *
     * @return string
     */
    public function getType()
    {
        return $this->_type;
    }

    /**
     * Get id.
     *
     * @return string
     */
    public function getId()
    {
        return $this->_id;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->_name;
    }

    /**
     * Detect is plan is default by comparing plan name.
     *
     * Todo: possibly enhance by setting env flag whenever
     * default plan is assigned or replaced. Comparing
     * names may be error prone in some instances, e.g. if
     * user explicitly selects cpanel plan or customizes plan
     * with default name.
     *
     * @return boolean
     */
    public function isDefault(): bool
    {
        // default plan name used to be "webcp".
        // Updated to avoid confusion with cpanel, as cpanel plans are named "webcp".
        return ($this->_name === Model_Backups_BackupPlan::DEFAULT_PLAN_NAME) || ($this->_name === 'webcp');
    }

    /**
     * @param string $instance_id
     * @return boolean
     */
    public function isProtectingResource($instance_id): bool
    {
        if (isset($this->_resource_ids)) {
            return in_array(strtolower($instance_id), $this->_resource_ids);
        }

        return false;
    }

    /**
     * check if policy obj returned by the api is of type policy.backup.machine
     *
     * @param array $policy
     * @return boolean
     */
    public function isBackupMachine(array $policy): bool
    {
        $type = $policy[self::TYPE];
        $is_backup_machine = $type === self::TYPE_POLICY_BACKUP_MACHINE;

        return $is_backup_machine;
    }

    /**
     * @param array  $policies
     * @param string $type
     * @return array
     */
    protected function _getPolicyByType(array $policies, $type)
    {
        foreach ($policies as $policy) {
            if ($policy[self::TYPE] === $type) {
                return $policy;
            }
        }
    }
}
