<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jul 09, 2021
 */

/**
 * @package InterWorx
 * @subpackage
 */
abstract class Model_Cache_Abstract
{
    const ID = 'id';
    const DBS = 'databases';
    const PLAN_ID = 'planId';
    const STATE = 'state';
    const UNIXUSER = 'unixuser';
    const ERROR = 'error';
    const TIME = 'time';
    const DIRECTORIES = 'directories';
    const UNMAPPED = 'unmapped';
    const DATE = 'date';
    const BACKUP_ID = 'backup_id';
    const API_DETAILS = 'api_details';
    const SW_USERS = 'sw_users';
    const TIMESTAMP = 'timestamp';
    const MAPPED_DATABASES = 'databases_mapped';
    const UNMAPPED_DATABASES = 'databases_unmapped';
    const DUMPS_DIR_UNMAPPED = 'dumps_dir_unmapped';
    const NAME = 'name';
    const SIZE = 'size';
    const HOST = 'host';

    const STATE_STARTED = 'started';
    const STATE_IN_PROGRESS = 'in progress';
    const STATE_CANCELLED = 'cancelled';
    const STATE_FAILED = 'failed';
    const STATE_COMPLETED = 'complete';



    /**
     * Constructor. Delete process cache to ensure that data is read from file.
     */
    public function __construct()
    {
        IW::Cache()->deleteAllProcess();
    }

    /**
     * @param string $id
     * @param array  $item
     */
    abstract public function add($id, array $item);

    /**
     * Check if cache is empty.
     *
     * @return boolean
     */
    public function empty()
    {
        $all = $this->getAll();
        return empty($all);
    }

    /**
     * check if item exists in cache.
     *
     * @param string $id
     */
    public function exists($id)
    {
        try {
            $cachedItem = $this->get($id);
            return !is_null($cachedItem);
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * Return value should be defined as a const in Model_Cache.
     * This will ensure that cache will be deleted correctly.
     *
     * @return string
     */
    abstract protected function _getCacheKey();

    /**
     * Get the name of the cache.
     *
     * @return string
     */
    abstract protected function _getCacheName();

    /**
     * Get cached item by  id.
     *
     * @param string $id
     */
    public function get($id)
    {
        $cached = $this->getAll();

        if (is_null($cached[$id])) {
            throw new Exception_Acronis_InvalidInput($id);
        }

        return $cached[$id];
    }

    /**
     * get cache by cache key.
     */
    public function getAll()
    {
        $cached = $this->_getCachedArray($this->_getCacheKey());

        if (is_null($cached)) {
            return array();
        }

        return $cached;
    }

    /**
     * Delete all items in cache.
     */
    public function deleteAll()
    {
        $this->_set(array());
    }


    /**
     * Format cached return object to assoc array.
     *
     * @return array
     */
    private function _getCachedArray()
    {
        $cachedObject = $this->_getCache();

        $assoc = true;
        return json_decode(json_encode($cachedObject), $assoc);
    }

    /**
     * get cache
     *
     * @return mixed
     */
    protected function _getCache()
    {
        $key = $this->_getCacheKey();
        return IW::Cache()->get($key);
    }

    /**
     * Set cache.
     *
     * @param array $cache
     */
    protected function _set(array $cache)
    {
        // make sure process cache
        // (in-memory rather than reading from file)
        // is being reset whenever cache is altered
        //IW::Cache()->deleteAllProcess();

        $key = $this->_getCacheKey();
        $this->_setCache($key, $cache);
    }

    /**
     * Set cache.
     * Default = set permanent
     *
     * @param string  $name
     * @param mixed   $data
     * @param integer $time
     */
    protected function _setCache($name, $data, $time = 0)
    {
        IW::Cache()->set($name, $data, $time);
    }

    /**
     * Updates item in cache
     *
     * @param string $id
     * @param mixed  $updatedItem
     */
    public function updateItem($id, $updatedItem)
    {
        $cached = $this->getAll();
        $cached[$id] = $updatedItem;
        $this->_set($cached);
    }


    /**
     * @param string $id
     */
    public function delete($id)
    {
        $all = $this->getAll();
        unset($all[$id]);

        $this->_set($all);
    }


    /**
     * @param string $id
     * @param string $function
     */
    protected function _validateId($id, $function)
    {
        $all = $this->getAll();
        if (!$all[$id]) {
            throw new Exception($function . ' failed: not cached: ' . $id);
        }
    }

    /**
     * @param array $itemToCache
     */
    protected function _validateNotEmpty(array $itemToCache)
    {
        if (empty($itemToCache)) {
            throw new Exception('Acronis Cache: Cannot cache empty value. Source: ' . $this->_getCacheName());
        }
    }

    /**
     * @param mixed $itemToCache
     */
    protected function _validateIsArray($itemToCache)
    {
        if (!is_array($itemToCache)) {
            throw new Exception('Acronis Cache: Expected array, got ' . gettype($itemToCache). '. Source: ' . $this->_getCacheName());
        }
    }

    /**
     * Validate that dbs contain keys as expected.
     *
     * @param array   $dbs
     * @param boolean $validateUnixuser
     */
    protected function _validateDbs(array $dbs, $validateUnixuser = true)
    {
        foreach ($dbs as $db) {
            $this->_validateIsArray($db);
            $this->_validateNotEmpty($db);
            $this->_validateDbsContainsKeys($db, $validateUnixuser);
        }
    }


    /**
     * @param array   $db
     * @param boolean $validateUnixuser
     */
    private function _validateDbsContainsKeys(array $db, $validateUnixuser = true)
    {
        $nameMissing = !array_key_exists(self::NAME, $db);
        $unixUserMissing = $validateUnixuser && !array_key_exists(self::UNIXUSER, $db);

        if ($nameMissing || $unixUserMissing) {
            throw new Exception('Acronis Cache: Cannot cache databases - missing required values');
        }
    }
}
