<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Sep 16, 2021
 */

/**
 * @package InterWorx
 * @subpackage
 */
abstract class AcronisTestCase extends IWorx_Framework_TestCase
{
    // so much common functionality - e.g. runBackup - should be shared between tests!
    const PARENT_DIR_PATH_TEST = '/home/acronis_recovery_test';
    const DIR_CONTENT_EXPECTED_COUNT = 400;
    const PLAN_NO_PRESCRIPT = 'NOT DEFAULT'; // MAKE SURE THIS EXISTS IN CLOUD
    const ACRONISTESTUSER = 'acronistest';

    private $_dbConnection;

    /**
     * Send ctrl request.
     *
     * @param string $action
     * @param array  $args
     */
    abstract protected function _sendCtrlRequest($action, array $args = array());

    /**
     * @return array
     */
    protected function _getBackups()
    {
        $Reply = $this->_sendCtrlRequest('listBackups');
        $this->assertReplySuccessful($Reply);

        $backups = $Reply->getPayloadData();

        if ($backups === null) {
            $this->fail('BACKUPS NULL');
        }

        return $backups;
    }

    /**
     * Check if getBackups is empty
     *
     * @return boolean
     */
    protected function _backupListEmpty()
    {
        return count($this->_getBackups()) == 0;
    }

    /**
     * @return Plugin_Acronis
     */
    protected function _getAcronisPlugin()
    {
        return IW::PluginManager()->getPlugin('acronis');
    }

    /**
     * @return boolean
     */
    protected function _isActivated()
    {
        $BackupPlan = new Model_Backups_BackupPlan();
        return $BackupPlan->isProtected();
    }

    /**
     * await restore.
     *
     * @param string $planId
     */
    protected function _awaitRestore($planId)
    {
        if (empty($planId)) {
            $this->fail('Acronis: ' . __FUNCTION__ . ': No plan id provided!');
        }
        // let acronis api update
        sleep(10);

        $attempts = 0;
        $max = 5;

        while (!$this->_isComplete($planId) && $attempts < $max) {
            $attempts++;
            sleep(60);
        }

        if (!$this->_isComplete($planId)) {
            $this->fail('Restore never completed - planId: ' . $planId);
        }
    }

    /**
     * Tests getRecoveryProgress
     *
     * @param string $planId
     */
    private function _isComplete($planId)
    {
        $Reply = $this->_getRecoveryProgress($planId);
        $this->assertReplySuccessful($Reply, __FUNCTION__ . 'EXPECTS SUCCESSFUL REPLY FOR GET RECOVERY PROGRESS!!! PLAN ID: ' . $planId);

        $payloadData = $Reply->getPayloadData();

        $this->assertArrayHasKey('progress', $payloadData);
        $this->assertArrayHasKey('state', $payloadData);
        $this->assertArrayHasKey('codeType', $payloadData);
        $this->assertArrayHasKey('codeError', $payloadData);
        $this->assertArrayHasKey('recoveryId', $payloadData);
        $this->assertArrayHasKey('planId', $payloadData);

        return $payloadData['state'] === 'completed';
    }

    /**
     * @param string $planId
     * @return Reply
     */
    protected function _getRecoveryProgress($planId)
    {
        $args = array(Form_AcronisBackups_RecoveryProgress::INPUT_PLAN_ID => $planId);
        return $this->_sendCtrlRequest('getRecoveryProgress', $args);
    }

    /**
     * @param string $action
     * @param array  $args
     * @return Reply
     */
    protected function _sendNwCtrlRequest($action, array $args = array())
    {
        return IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Acronis',
            $action,
            $args,
            new NodeWorx(1)
        );
    }

    /**
     * @param string $action
     * @param array  $args
     * @param string $username
     */
    protected function _sendSwCtrlRequest($action, array $args, $username)
    {
        return IW::FC()->routeFromPHP(
            'Ctrl_Siteworx_Acronis',
            $action,
            $args,
            $this->_getSw($username)
        );
    }

    /**
     * Get backups.
     *
     * @return Model_Backups_Backup
     */
    protected function _getBackup()
    {
        // cache SHOULD delete from process, (see IW::Cache() _set)
        // but somehow it doesn't always when running tests.
        // Deleting process data here instead.
        IW::Cache()->deleteAllProcess();

        $backups = $this->_getBackups();
        $backup = $backups[0];

        if (is_null($backup)) {
            return null;
        }
        return new Model_Backups_Backup($backup);
    }

    /**
     * Get SW account acronistest.
     *
     * @param string $user
     * @return SiteWorx
     */
    protected function _getSw($user = self::ACRONISTESTUSER)
    {
        $SW = new SiteWorx();
        $SW->populateUniqname($user);

        return $SW;
    }


    /**
     * Run backup & await api.
     */
    protected function _runBackup()
    {
        $OldBackup = $this->_getBackup();
        $Reply = $this->_sendNwCtrlRequest('runBackup');
        $this->assertReplySuccessful($Reply);
        if (is_null($OldBackup)) {
            $this->_awaitInitialBackup();
        } else {
            $this->_awaitBackup($OldBackup->getBackupId());
        }
    }


    /**
     * Await acronis api to include the newly created backup.
     * Wait for max 5 minutes.
     *
     * @param string $oldBackupId
     */
    protected function _awaitBackup($oldBackupId)
    {
        $maxAttempts = 10;
        $attempts = 0;

        while ($this->_newestBackupIdSameAs($oldBackupId) && $attempts < $maxAttempts) {
            sleep(60);
            $attempts++;
        }

        if ($this->_getBackup()->getBackupId() === $oldBackupId) {
            $this->fail("API DOES NOT INCLUDE BACKUP AFTER $maxAttempts MIN");
        }
    }

    /**
     * @return void
     */
    protected function _awaitInitialBackup()
    {
        $maxAttempts = 10;
        $attempts = 0;

        while ($this->_backupListEmpty() && $attempts < $maxAttempts) {
            sleep(60);
            $attempts++;
        }

        if ($this->_backupListEmpty()) {
            $this->fail("API DOES NOT INCLUDE BACKUP AFTER $maxAttempts MIN");
        }
    }


    /**
     * Compare newest backup id to old backup id to see if api is updated.
     *
     * @param string $oldBackupId
     */
    protected function _newestBackupIdSameAs($oldBackupId)
    {
        $newestId = $this->_getBackup()->getBackupId();
        return $newestId === $oldBackupId;
    }


    /**
     * @param array   $dirs
     * @param boolean $deleteNewContent
     * @return Reply
     */
    protected function _recoverDirectories(array $dirs, $deleteNewContent = false)
    {
        $Backup = $this->_getBackup();
        $args = array(
            Form_AcronisBackups::INPUT_MULTI_DIR => $dirs,
            Form_AcronisBackups::INPUT_BACKUP_ID => $Backup->getBackupId(),
            Form_AcronisBackups::INPUT_DELETE_FILES => $deleteNewContent
        );

        return $this->_sendCtrlRequest('recoverDirectories', $args);
    }

    /**
     * await restore.
     *
     * @param string $planId
     */
    protected function _awaitPaused($planId)
    {
        // let acronis api update
        sleep(10);

        $attempts = 0;
        $max = 5;

        while (!$this->_isPaused($planId) && $attempts < $max) {
            $attempts++;
            sleep(60);
        }

        if (!$this->_isPaused($planId)) {
            $this->fail('not getting paused state from getRecoveryProgress');
        }
    }

    /**
     * Tests getRecoveryProgress
     *
     * @param string $planId
     */
    private function _isPaused($planId)
    {
        $Reply = $this->_getRecoveryProgress($planId);
        $this->assertReplySuccessful($Reply, __FUNCTION__ . ' EXPECTS SUCCESSFUL REPLY FOR GET RECOVERY PROGRESS!!! PLAN ID: ' . $planId);

        $payloadData = $Reply->getPayloadData();
        return $payloadData['state'] === 'paused';
    }

    /**
     * @param string $planId
     * @return string
     */
    protected function _getRecoveryId($planId)
    {
        $ReplyRecoveryProgress = $this->_getRecoveryProgress($planId);
        $payloadDataRecoveryProgress = $ReplyRecoveryProgress->getPayloadData();
        return $payloadDataRecoveryProgress['recoveryId'];
    }

    /**
     * @param string $recoveryId
     * @return Reply
     */
    protected function _resumeRecovery($recoveryId)
    {
        $args = array(Form_AcronisBackups_ResumeRecovery::INPUT_RECOVERY_ID => $recoveryId);
        return $this->_sendCtrlRequest('resumeRecovery', $args);
    }

    /**
     * @param string $root
     * @return string
     */
    protected function _getRestoredPath($root)
    {
        $tempContent = Model_AcronisFileSys::getDirContent($root);
        $dateDir = $tempContent[0];

        $tempBase = "$root/$dateDir";
        $contentTempBase = Model_AcronisFileSys::getDirContent($tempBase);
        $recoveredFolder = $contentTempBase[0];

        return "$tempBase/$recoveredFolder";
    }

    /**
     * Delete flag "initiated"
     */
    protected function _deleteFlagInitiatedIfExists()
    {
        $initiated = IW::ENV()->getEnvFlag(Model_Recovery_Files::ENV_FLAG_RESTORE_INITIATED);
        if (isset($initiated)) {
            IW::ENV()->clearEnvFlag(Model_Recovery_Files::ENV_FLAG_RESTORE_INITIATED);
        }
    }

    /**
     * @param Reply $Reply
     */
    protected function _assertReplyReturnsRestoreInProgress(Reply $Reply)
    {
        $errors = $Reply->getErrors();
        $this->assertContains(AcronisReply::LANG_RECOVERY_ALREADY_IN_PROGRESS, $errors, $Reply->getErrorsAsString());
    }

    /**
     * @param Reply  $Reply
     * @param string $langKey
     */
    protected function _assertReplyReturnsError(Reply $Reply, $langKey)
    {
        $errors = $Reply->getErrors();
        $this->assertContains($langKey, $errors, $Reply->getErrorsAsString());
    }

    /**
     * @param Reply $Reply
     */
    protected function _assertReplyReturnsDbRestoreInProgress(Reply $Reply)
    {
        $errors = $Reply->getErrors();
        $this->assertContains(AcronisReply::LANG_DB_RECOVERY_ALREADY_IN_PROGRESS, $errors, $Reply->getErrorsAsString());
    }

    /**
     * Drop sw dbs.
     */
    protected function _dropSwDbs()
    {
        $SW = $this->_getAcronisTestSw();

        if ($SW->exists()) {
            $Command = new Command('/home/interworx/bin/php /home/interworx/plugins/acronis/bin/acronisdbseeder.php --dropSwDatabases');
            $Command->runAsRoot();
            $Command->exec();

            if (!$Command->wasSuccessful()) {
                // assume dbs don't exist
                IW::getLog()->warning('Acronis Integration tests: Could not drop SW dbs: ');
                IW::getLog()->warning($Command->getResultMessage());
            }
        }
    }

    /**
     * Get SW account acronistest.
     *
     * @return SiteWorx
     */
    protected function _getAcronisTestSw()
    {
        $SW = new SiteWorx();
        $SW->populateUniqname(self::ACRONISTESTUSER);

        return $SW;
    }

    /**
     * set env flag 'initiated'
     */
    protected function _setInitiated()
    {
        IW::ENV()->setEnvFlag(Model_Recovery_Files::ENV_FLAG_RESTORE_INITIATED);
    }

    /**
     * clear env flag 'initiated'
     */
    protected function _clearInitiated()
    {
        IW::ENV()->clearEnvFlag(Model_Recovery_Files::ENV_FLAG_RESTORE_INITIATED);
    }

    /**
     * @return boolean
     */
    protected function _dbRestoreInProgress()
    {
        IW::Cache()->deleteAllProcess();

        foreach ($this->_getDbRecoveries() as $recovery) {
            if ($recovery['state'] === 'started' || $recovery['state'] === 'in progress') {
                return true;
            }
        }

        return false;
    }


    /**
     * get db recoveries by sending req to listDatabaseRecoveries endpoint.
     *
     * @return array
     */
    protected function _getDbRecoveries()
    {
        $Reply = $this->_sendCtrlRequest('listDatabaseRecoveries');
        return $Reply->getPayloadData();
    }

    /**
     * Skip test if db restore not in progress
     */
    protected function _skipIfNotInProgress()
    {
        if (!$this->_dbRestoreInProgress()) {
            sleep(5);
            if (!$this->_dbRestoreInProgress()) {
                $this->markTestSkipped('Unable to test! restore not in progress');
            }
        }
    }


    /**
     * Await db recovery
     */
    protected function _awaitDbRecoveries()
    {
        $max = 5;
        $attempts = 0;

        while ($this->_dbRestoreInProgress() && $attempts < $max) {
            $attempts++;
            sleep(60);
        }

        if ($this->_dbRestoreInProgress()) {
            $this->fail('db restore still in progress');
        }
    }

    /**
     * Await db recovery with specified id.
     *
     * @param string $id
     */
    protected function _awaitDbRecovery($id)
    {
        $attempts = 0;
        $max = 6;

        while ($attempts < $max && !$this->_dbRecoveryComplete($id)) {
            $attempts++;
            sleep(10);
        }
    }

    /**
     * @param string $id
     * @return boolean
     */
    private function _dbRecoveryComplete($id)
    {
        $Reply = $this->_getDbRecovery($id);
        $payloadData = $Reply->getPayloadData();
        return $payloadData['state'] === 'complete';
    }

    /**
     * @param string $id
     * @return Reply
     */
    private function _getDbRecovery($id)
    {
        $args = array('id' => $id);
        return $this->_sendCtrlRequest('getDatabaseRecoveryState', $args);
    }

    /**
     * make sure files recovered to temp location
     *
     * @param string  $location
     * @param array   $restoredDbs
     * @param boolean $unmapped
     */
    protected function _assertDbFilesRestoredToTemp($location, array $restoredDbs, $unmapped = false)
    {
        $this->assertTrue(Model_AcronisFileSys::isDir($location), 'RECOVERY FOLDER DOES NOT EXIST AFTER RESTORE TO ALT LOCATION! ' . $location);

        foreach ($restoredDbs as $dbName) {
            $dumpsPath = "$location/$dbName";
            $this->assertTrue(Model_AcronisFileSys::isDir($dumpsPath), 'NOT A DIRECTORY: ' . $dumpsPath);

            $table = $this->_getTableDumpName($dbName, $unmapped);
            $schemaOnly = "$dbName-schema-only.sql";

            $dumps = Model_AcronisFileSys::getDirContent($dumpsPath);
            $this->assertContains($table, $dumps, 'Restored dumps does not contain expected file: ' . $table);
            $this->assertContains($schemaOnly, $dumps, 'Restored dumps does not contain expected file: ' . $schemaOnly);
        }
    }

    /**
     * @param string  $dbName
     * @param boolean $unmapped
     */
    private function _getTableDumpName($dbName, $unmapped = false)
    {
        $suffix = array_pop(explode('_', $dbName));
        $tableName = $unmapped ? 'systemdb' : "swtable_$suffix";
        return "$dbName.$tableName.sql.gz";
    }


    /**
     * Make sure dbs exists & are populated after restore.
     */
    protected function _assertSwDbsRestored()
    {
        $restoredDbs = array('testdb1', 'testdb2');

        foreach ($restoredDbs as $dbName) {
            $this->_assertAcronistestDbPopulated($dbName);
        }
    }


    /**
     * @param string $dbNameSuffix
     */
    private function _assertAcronistestDbPopulated($dbNameSuffix)
    {
        $dbName = "acronistest_$dbNameSuffix";
        $this->_connectDb($dbName);

        $tableName = "swtable_$dbNameSuffix";
        $this->_assertIdPopulatedInTable($tableName);
    }


    /**
     * Connect to db.
     *
     * @param string $dbName
     */
    protected function _connectDb($dbName)
    {
        $dsn = substr(Ini::get(Ini::MYSQL, 'rootdsn'), 0, -5) . $dbName;
        $dbConnect = IWorxDbUtil::db_connect($dsn, false);

        if (get_class($dbConnect) === DB_Error::class) {
            $this->fail('unable to connect to db ' . $dbName);
        }

        $this->_dbConnection = $dbConnect;
    }

    /**
     * @return object
     */
    protected function _getDbConnection()
    {
        return $this->_dbConnection;
    }

    /**
     * @param string $tableName
     */
    protected function _assertIdPopulatedInTable($tableName)
    {
        $sql = "SELECT id FROM $tableName";

        $result = IWorxDbUtil::db_query($this->_dbConnection, $sql);

        $row1 = $result->fetchRow(DB_FETCHMODE_ASSOC, 0);
        $row2 = $result->fetchRow(DB_FETCHMODE_ASSOC, 1);
        $row3 = $result->fetchRow(DB_FETCHMODE_ASSOC, 2);

        $this->assertEquals(1, $row1['id']);
        $this->assertEquals(2, $row2['id']);
        $this->assertEquals(3, $row3['id']);
    }

    /**
     * @param string $user
     */
    protected function _deleteUserDomainFolder($user)
    {
        $SW = $this->_getSw($user);
        $root = $SW->getPackageRoot();
        Model_AcronisFileSys::rm("$root/$user.com");
    }


    /**
     * Await restores.
     */
    protected function _awaitRestores()
    {
        $attempts  = 0;
        $max = 10;

        while ($this->_restoreInProgress() && $attempts < $max) {
            $attempts++;
            sleep(60);
        }
    }

    /**
     * Check if restore is in progress.
     */
    private function _restoreInProgress()
    {
        $Reply = $this->_sendNwCtrlRequest('recoveryInProgress');
        $payloadData = $Reply->getPayloadData();
        return $payloadData['recoveryInProgress'];
    }
}
