<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2022 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jan 14, 2022
 */

require_once(__DIR__ . '/../../Nodeworx.php');

/**
 * @package InterWorx
 * @subpackage
 */
class RegeneratePrescriptDataTest extends Acronis_NodeworxTest
{
    const CACHE_PATH = '/usr/local/interworx/var/cache/IWorx_Cache/backup.acronis.cache';

    const CACHE_SW_USERS = 'sw_users';
    const CACHE_DUMPS_DIR = 'dumps_dir_unmapped';
    const CACHE_UNMAPPED_DBS = 'databases_unmapped';
    const CACHE_MAPPED_DBS = 'databases_mapped';
    /**
     * test api endpoint regeneratePrescriptDataAction
     */
    public function testRegeneratePrescriptData()
    {
        echo __FUNCTION__;

        $backup_pre_delete_prescript = $this->_getLatestCacheBackup();

        $this->_overwriteLatestBackup();
        $this->_assertPrescriptDeleted();
        $this->_sendCtrlRequest('regeneratePrescriptData');
        $this->_assertPrescriptDataRestored($backup_pre_delete_prescript);
    }

    /**
     * Ensure updated backup contains the right prescript data.
     *
     * @param array $backup_pre_delete_prescript
     */
    private function _assertPrescriptDataRestored(array $backup_pre_delete_prescript)
    {
        $updated_backup = $this->_getLatestCacheBackup();

        // ensure prescript data has been added
        $this->assertContains(self::CACHE_SW_USERS, array_keys($updated_backup));
        $this->assertContains(self::CACHE_DUMPS_DIR, array_keys($updated_backup));
        $this->assertContains(self::CACHE_UNMAPPED_DBS, array_keys($updated_backup));
        $this->assertContains(self::CACHE_MAPPED_DBS, array_keys($updated_backup));

        // ensure prescript data contains right values
        $this->assertSame($backup_pre_delete_prescript[self::CACHE_SW_USERS], $updated_backup[self::CACHE_SW_USERS]);
        $this->assertSame($backup_pre_delete_prescript[self::CACHE_MAPPED_DBS], $updated_backup[self::CACHE_MAPPED_DBS]);
        $this->assertSame($backup_pre_delete_prescript[self::CACHE_UNMAPPED_DBS], $updated_backup[self::CACHE_UNMAPPED_DBS]);
        $this->assertSame($backup_pre_delete_prescript[self::CACHE_DUMPS_DIR], $updated_backup[self::CACHE_DUMPS_DIR]);
    }

    /**
     * Make sure latest backup in cache is lacking prescript data.
     */
    private function _assertPrescriptDeleted()
    {
        $backup = $this->_getLatestCacheBackup();

        // ensure no prescript data
        $this->assertNotContains(self::CACHE_SW_USERS, array_keys($backup));
        $this->assertNotContains(self::CACHE_DUMPS_DIR, array_keys($backup));
        $this->assertNotContains(self::CACHE_UNMAPPED_DBS, array_keys($backup));
        $this->assertNotContains(self::CACHE_MAPPED_DBS, array_keys($backup));
    }

    /**
     * get latest backup from cache.
     *
     * @return array
     */
    private function _getLatestCacheBackup()
    {
        $cache_content = IWorx_Json::readFromFile(self::CACHE_PATH, true);
        return array_pop($cache_content['value']);
    }

    /**
     * Overwrite latest backup in cache.
     *
     * Will exclude prescript data unless passed in as param.
     *
     * @param array $prescript_data
     */
    private function _overwriteLatestBackup(array $prescript_data = array())
    {
        $cache_content = IWorx_Json::readFromFile(self::CACHE_PATH, true);
        $last_item = array_pop($cache_content['value']);
        $backup_id = $last_item['backup_id'];
        $api_details = $last_item['api_details'];

        $updated_backup = array(
            'backup_id' => $backup_id,
            'api_details' => $api_details
        );

        $cache_content['value'][$backup_id] = array_merge($updated_backup, $prescript_data);
        IWorx_Json::writeToFile($cache_content, self::CACHE_PATH);
    }

    /**
     * test api endpoint recreateBackupCacheAction
     */
    public function testRecreateBackupCacheShouldCacheNewBackupAndGeneratePrescriptData()
    {
        echo __FUNCTION__;

        $backup_pre_delete = $this->_getLatestCacheBackup();

        $this->_deleteLatestBackup();
        $this->assertNotSame($backup_pre_delete, $this->_getLatestCacheBackup());

        $Reply = $this->_sendCtrlRequest('recreateBackupCache', array('ab_backup_id' => $backup_pre_delete['backup_id']));

        if (!$Reply->wasSuccessful()) {
            $this->fail('Action failed! ' . $Reply->getErrorsAsString());
        }

        $this->assertSame($backup_pre_delete, $this->_getLatestCacheBackup());
    }

    /**
     * test api endpoint recreateBackupCacheAction
     */
    public function testRecreateBackupCacheShouldAddPrescript()
    {
        echo __FUNCTION__;

        $backup_pre_delete = $this->_getLatestCacheBackup();

        $this->_overwriteLatestBackup();
        $this->_assertPrescriptDeleted();

        $Reply = $this->_sendCtrlRequest('recreateBackupCache', array('ab_backup_id' => $backup_pre_delete['backup_id']));

        if (!$Reply->wasSuccessful()) {
            $this->fail('Action failed! ' . $Reply->getErrorsAsString());
        }

        $this->assertSame($backup_pre_delete, $this->_getLatestCacheBackup());
    }

    /**
     * test api endpoint recreateBackupCacheAction
     */
    public function testRecreateBackupCacheShouldOverwritePrescript()
    {
        echo __FUNCTION__;

        $backup_pre_delete = $this->_getLatestCacheBackup();

        $this->_overwriteLatestBackup(array(
            self::CACHE_SW_USERS => []
        ));

        $this->assertEmpty($this->_getLatestCacheBackup()[self::CACHE_SW_USERS]);

        $Reply = $this->_sendCtrlRequest('recreateBackupCache', array('ab_backup_id' => $backup_pre_delete['backup_id']));
        if (!$Reply->wasSuccessful()) {
            $this->fail('Action failed! ' . $Reply->getErrorsAsString());
        }

        $updated_backup = $this->_getLatestCacheBackup();
        $this->assertSame($updated_backup[self::CACHE_SW_USERS], $backup_pre_delete[self::CACHE_SW_USERS]);
        $this->assertSame($updated_backup[self::CACHE_MAPPED_DBS], $backup_pre_delete[self::CACHE_MAPPED_DBS]);
        $this->assertSame($updated_backup[self::CACHE_UNMAPPED_DBS], $backup_pre_delete[self::CACHE_UNMAPPED_DBS]);
        $this->assertSame($updated_backup[self::CACHE_DUMPS_DIR], $backup_pre_delete[self::CACHE_DUMPS_DIR]);
    }


    /**
     * delete latest backup from cache.
     */
    private function _deleteLatestBackup()
    {
        $cache_content = IWorx_Json::readFromFile(self::CACHE_PATH, true);
        $last_item = array_pop($cache_content['value']);
        $backup_id = $last_item['backup_id'];

        unset($cache_content['value'][$backup_id]);
        IWorx_Json::writeToFile($cache_content, self::CACHE_PATH);
    }
}
