<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Sep 20, 2021
 */
require_once(__DIR__ . '/../../Siteworx.php');
/**
 * @package InterWorx
 * @subpackage
 */
class SwRestoreDatabasesTest extends Acronis_SiteworxTest
{
    private $_dbConnection;

    /**
     * Disconnect db if connection created.
     */
    public function tearDown()
    {
        $this->_deleteFlagInitiatedIfExists();

        if (isset($this->_dbConnection)) {
            $this->_dbConnection->disconnect();
        }
    }


    /**
     * test restoreDatabases - second req should fail, first one should be set in progress.
     */
    public function testRestoreDatabases()
    {
        $this->_awaitDbRecoveries();
        $this->_dropSwDbs();

        $dbs = array('acronistest_testdb1', 'acronistest_testdb2');
        $SuccessReply = $this->_restoreDbs($dbs);
        $this->assertReplySuccessful($SuccessReply);

        $payloadData = $SuccessReply->getPayloadData();
        $id = $payloadData['recoveryId'];

        $this->_assertRestoreFailsWhenDbRestoreInProgress();
        $this->_awaitDbRecovery($id);
        $this->_assertSwDbsRestored();
        $this->_assertDbRecoveriesIncludesId($id);
    }

    /**
     * @param string $id
     * @return boolean
     */
    private function _assertDbRecoveriesIncludesId($id)
    {
        $Reply = $this->_sendCtrlRequest('listDatabaseRecoveries');
        $recoveries = $Reply->getPayloadData();

        foreach ($recoveries as $recovery) {
            if ($recovery['id'] === $id) {
                return true;
            }
        }

        return false;
    }

    /**
     * Expects request to fail when db restore is in progress
     */
    private function _assertRestoreFailsWhenDbRestoreInProgress()
    {
        sleep(2);
        $this->_skipIfNotInProgress();

        $dbs = array('acronistest_testdb1', 'acronistest_testdb2');
        $FailReply = $this->_restoreDbs($dbs);
        $this->assertReplyNotSuccessful($FailReply, 'Second request should fail while first is in progress!');
        $this->_assertReplyReturnsDbRestoreInProgress($FailReply);
    }

    /**
     * test restoreDatabases
     */
    public function testRestoreDatabasesShouldFailIfDbDoesNotExist()
    {
        $this->_dropSwDbs();
        $dbs = array('dbdoesnotexist');
        $Reply = $this->_restoreDbs($dbs);

        $this->assertReplyNotSuccessful($Reply, 'Database does not exist! Request should fail');
        $this->_assertReplyReturnsDbDoesNotExist($Reply);
    }

    /**
     * @param Reply $Reply
     */
    private function _assertReplyReturnsDbDoesNotExist(Reply $Reply)
    {
        $errors = $Reply->getErrors();
        $this->assertContains(AcronisReply::LANG_DB_DOES_NOT_EXIST, $errors);
    }

    /**
     * @param array $dbs
     */
    private function _restoreDbs(array $dbs)
    {
        $Backup = $this->_getLatestSwBackup(self::ACRONISTESTUSER);
        $args = array(
            Form_AcronisBackups::INPUT_BACKUP_ID => $Backup->getBackupId(),
            Form_AcronisBackups::INPUT_DBS => $dbs
        );

        return $this->_sendCtrlRequest('restoreDatabases', $args);
    }

    /**
     * test restoreDatabases - should fail when flag inititated is set
     */
    public function testRestoreDatabasesShouldFailWhenRestoreInitiated()
    {
        $this->_dropSwDbs();
        $dbs = array('acronistest_testdb1');

        $this->_setInitiated();

        $Reply = $this->_restoreDbs($dbs);
        $this->assertReplyNotSuccessful($Reply, 'Other restore initiated - req should fail!');
        $this->_assertReplyReturnsRestoreInProgress($Reply);

        $this->_clearInitiated();
    }
}
