<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Sep 20, 2021
 */
require_once(__DIR__ . '/../../Siteworx.php');
/**
 * @package InterWorx
 * @subpackage
 */
class SwRestoreDatabasesAndDirectoriesTest extends Acronis_SiteworxTest
{
    /**
     * Clear 'initiated' flag if leftover from tests.
     */
    public function tearDown()
    {
        $this->_deleteFlagInitiatedIfExists();
    }

    /**
     * test restoreDatabasesAndDirectories - should fail with non existing dbs
     */
    public function testRestoreDatabasesAndDirectoriesShouldFailWithInvalidDbs()
    {
        echo __FUNCTION__;

        $this->_awaitDbRecoveries();
        $this->_awaitRestores();

        $dbs = array('acronistest_testdb1','acrtest_testdb2');
        $Reply = $this->_restoreDatabasesAndDirectories(array(), $dbs);
        $this->assertReplyNotSuccessful($Reply, 'should fail - invalid dbs');
        $this->_assertReplyReturnsError($Reply, AcronisReply::LANG_DB_DOES_NOT_EXIST);
    }

    /**
     * test restoreDatabasesAndDirectories - should fail with invalid dir paths
     */
    public function testRestoreDatabasesAndDirectoriesShouldFailWithInvalidDirs()
    {
        echo __FUNCTION__;

        $this->_awaitDbRecoveries();
        $this->_awaitRestores();

        $dirs = array('SOMEDIRDOESNOTEXIST');
        $Reply = $this->_restoreDatabasesAndDirectories($dirs, array());
        $this->assertReplyNotSuccessful($Reply, 'should fail - invalid path');
        $this->_assertReplyReturnsError($Reply, AcronisReply::LANG_INVALID_PATH);
    }

    /**
     * test restoreDatabasesAndDirectories - should fail when args missing
     */
    public function testRestoreDatabasesAndDirectoriesShouldFailNoArgs()
    {
        echo __FUNCTION__;

        $this->_awaitDbRecoveries();
        $this->_awaitRestores();

        $Reply = $this->_restoreDatabasesAndDirectories(array(), array());
        $this->assertReplyNotSuccessful($Reply, 'should fail - invalid input');
        $this->_assertReplyReturnsError($Reply, AcronisReply::LANG_INVALID_INPUT);
    }

    /**
     * test restoreDatabasesAndDirectories - should succeed
     */
    public function testRestoreDatabasesAndDirectoriesShouldSucceed()
    {
        echo __FUNCTION__;

        $this->_awaitDbRecoveries();
        $this->_awaitRestores();

        $this->_dropSwDbs();
        $this->_deleteUserDomainFolder(self::ACRONISTESTUSER);

        $absolutePath = $this->_getSwAbsolutePath(self::ACRONISTESTUSER, '/acronistest.com');
        $this->assertFalse(Model_AcronisFileSys::isDir($absolutePath));

        $dirs  = array('/acronistest.com');
        $dbs = array('acronistest_testdb1', 'acronistest_testdb2');

        $Reply = $this->_restoreDatabasesAndDirectories($dirs, $dbs);
        $this->assertReplySuccessful($Reply);

        $payloadData = $Reply->getPayloadData();
        $dirsResult = $payloadData['directories'];
        $dbResult = $payloadData['databases'];

        $this->_awaitDbRecovery($dbResult['restoreId']);
        $this->_awaitRestore($dirsResult['planId']);

        $this->_assertSwDbsRestored();
        $this->assertTrue(Model_AcronisFileSys::isDir($absolutePath));
    }


    /**
     * test restoreDatabasesAndDirectories - should fail when flag 'initiated' is set
     */
    public function testRestoreDatabasesAndDirectoriesShouldFailWhenInitiated()
    {
        echo __FUNCTION__;

        $this->_setInitiated();
        $dirs  = array('/acronistest.com');
        $dbs = array('acronistest_testdb1', 'acronistest_testdb2');

        $Reply = $this->_restoreDatabasesAndDirectories($dirs, $dbs);
        $this->assertReplyNotSuccessful($Reply, 'Should fail when initiated set');
        $this->_assertReplyReturnsError($Reply, AcronisReply::LANG_RECOVERY_ALREADY_IN_PROGRESS);
        $this->_clearInitiated();
    }

    /**
     * @param array $dirs
     * @param array $dbs
     * @return Reply
     */
    private function _restoreDatabasesAndDirectories(array $dirs, array $dbs)
    {
        $Backup = $this->_getLatestSwBackup(self::ACRONISTESTUSER);
        $backupId = $Backup->getBackupId();

        $args = array(
            Form_AcronisBackups::INPUT_BACKUP_ID => $backupId,
            Form_AcronisBackups::INPUT_DBS => $dbs,
            Form_AcronisBackups::INPUT_MULTI_DIR => $dirs
        );

        return $this->_sendCtrlRequest('restoreDatabasesAndDirectories', $args);
    }
}
