<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jun 16, 2021
 */
require_once(__DIR__ . './../../../Model/Cache/Restore/Databases.php');
require_once(__DIR__ . './../../../Model/Cache/Abstract.php');

/**
 * @package InterWorx
 * @subpackage TESTS
 */
class Cache_DbRestoresTest extends IWorx_Framework_TestCase
{
    private $_id = 'somerandomid';
    private $_dbs = array('somedb', 'anotherdb');
    private $_unixUser = 'someunixuser';
    private $_SUT;
    private $_timestamp;

    /**
     * Create SUT & add to cache
     */
    public function setUp()
    {
        $this->_SUT = new Model_Cache_Restore_Databases();
        $this->_timestamp = time();

        $data = array(
            $this->_SUT::TIME => Model_Acronis::getDateString($this->_timestamp),
            $this->_SUT::DBS => $this->_dbs,
            $this->_SUT::ID => $this->_id,
            $this->_SUT::UNIXUSER => $this->_unixUser,
        );

        $this->_SUT->add($this->_id, $data);
    }

    /**
     * Delete test item from cache
     */
    public function tearDown()
    {
        $this->_getSut()->deleteAll();
    }

    /**
     * Test add, addDatabases, and addUnixUser (in setUp)
     */
    public function testAdd()
    {
        $SUT = $this->_getSut();

        $actual = $SUT->get($this->_id);

        $this->assertArrayHasKey($SUT::TIME, $actual);
        $this->assertArrayHasKey($SUT::ID, $actual);
        $this->assertArrayHasKey($SUT::DBS, $actual);
        $this->assertArrayHasKey($SUT::UNIXUSER, $actual);

        $this->assertSame($actual[$SUT::ID], $this->_id);
        $this->assertSame($actual[$SUT::DBS], $this->_dbs);
        $this->assertSame($actual[$SUT::UNIXUSER], $this->_unixUser);
    }

    /**
     * Should throw exception when not exists.
     */
    public function testSetStartedShouldThrowException()
    {
        $SUT = $this->_getSut();

        $this->expectException(Exception::class);
        $SUT->setStarted('nonexistingid');
    }

    /**
     * Should update state
     */
    public function testSetStartedShouldUpdateState()
    {
        $SUT = $this->_getSut();

        $actual = $SUT->get($this->_id);
        $expected = $SUT::STATE_STARTED;
        $this->assertNotSame($expected, $actual[$SUT::STATE]);

        $SUT->setStarted($this->_id);
        $actual = $SUT->get($this->_id);
        $this->assertSame($expected, $actual[$SUT::STATE]);
    }


    /**
     * Should throw exception when not exists.
     */
    public function testSetInProgressShouldThrowException()
    {
        $SUT = $this->_getSut();

        $this->expectException(Exception::class);
        $SUT->setInProgress('nonexistingid');
    }

    /**
     * Should update state
     */
    public function testSetInProgressShouldUpdateState()
    {
        $SUT = $this->_getSut();

        $actual = $SUT->get($this->_id);
        $expected = $SUT::STATE_IN_PROGRESS;
        $this->assertNotSame($expected, $actual[$SUT::STATE]);

        $SUT->setInProgress($this->_id);
        $actual = $SUT->get($this->_id);
        $this->assertSame($expected, $actual[$SUT::STATE]);
    }


    /**
     * Should throw exception when not exists.
     */
    public function testSetCancelledThrowException()
    {
        $SUT = $this->_getSut();

        $this->expectException(Exception::class);
        $SUT->setCancelled('nonexistingid');
    }

    /**
     * Should update state
     */
    public function testCancelledShouldUpdateState()
    {
        $SUT = $this->_getSut();

        $actual = $SUT->get($this->_id);
        $expected = $SUT::STATE_CANCELLED;
        $this->assertNotSame($expected, $actual[$SUT::STATE]);

        $SUT->setCancelled($this->_id);
        $actual = $SUT->get($this->_id);
        $this->assertSame($expected, $actual[$SUT::STATE]);
    }

    /**
     * Should throw exception when not exists.
     */
    public function testSetCompleteShouldThrowException()
    {
        $SUT = $this->_getSut();

        $this->expectException(Exception::class);
        $SUT->setComplete('nonexistingid');
    }

    /**
     * Should update state
     */
    public function testSetCompleteShouldUpdateState()
    {
        $SUT = $this->_getSut();

        $actual = $SUT->get($this->_id);
        $expected = $SUT::STATE_COMPLETED;
        $this->assertNotSame($expected, $actual[$SUT::STATE]);

        $SUT->setComplete($this->_id);
        $actual = $SUT->get($this->_id);
        $this->assertSame($expected, $actual[$SUT::STATE]);
    }


    /**
     * Should throw exception when not exists.
     */
    public function testSetFailedShouldThrowException()
    {
        $SUT = $this->_getSut();

        $this->expectException(Exception::class);
        $SUT->setFailed('nonexistingid');
    }

    /**
     * Should update state
     */
    public function testSetFailedShouldUpdateState()
    {
        $SUT = $this->_getSut();

        $actual = $SUT->get($this->_id);
        $expected = $SUT::STATE_FAILED;
        $this->assertNotSame($expected, $actual[$SUT::STATE]);

        $SUT->setFailed($this->_id);
        $actual = $SUT->get($this->_id);
        $this->assertSame($expected, $actual[$SUT::STATE]);
    }

    /**
     *
     */
    public function testSetPlanIdShouldThrowException()
    {
        $SUT = $this->_getSut();

        $this->expectException(Exception::class);
        $SUT->setPlanId('nonexistingid', 'planId');
    }

    /**
     *
     */
    public function testSetPlanIdShouldUpdatePlanId()
    {
        $SUT = $this->_getSut();

        $actual = $SUT->get($this->_id);
        $expected = 'randomplanid';

        $this->assertNotSame($expected, $actual[$SUT::PLAN_ID]);

        $SUT->setPlanId($this->_id, $expected);
        $actual = $SUT->get($this->_id);
        $this->assertSame($expected, $actual[$SUT::PLAN_ID]);
    }

    /**
     *
     */
    public function testSetErrorMsgShouldThrowException()
    {
        $SUT = $this->_getSut();

        $this->expectException(Exception::class);
        $SUT->setErrorMsg('nonexistingid', 'error');
    }

    /**
     *
     */
    public function testSetErrorMsgShouldUpdateErrorMsg()
    {
        $SUT = $this->_getSut();

        $actual = $SUT->get($this->_id);
        $expected = 'error msg!';

        $this->assertNotSame($expected, $actual[$SUT::ERROR]);

        $SUT->setErrorMsg($this->_id, $expected);
        $actual = $SUT->get($this->_id);
        $this->assertSame($expected, $actual[$SUT::ERROR]);
    }

    /**
     * test getsByUnixUser should return expected amount.
     */
    public function testGetRestoresByUnixUser()
    {
        $SUT = $this->_getSut();
        $actual = $SUT->getRestoresByUnixUser($this->_unixUser);
        $this->assertCount(1, $actual);
    }

    /**
     * test userHasActiveRestores
     * should return true when restore set to in progress
     * and false otherwise
     */
    public function testUserHasActiveRestores()
    {
        $SUT = $this->_getSut();

        $SUT->setInProgress($this->_id);
        $this->assertTrue($SUT->userHasActiveRestores($this->_unixUser));

        $SUT->setComplete($this->_id);
        $this->assertFalse($SUT->userHasActiveRestores($this->_unixUser));

        $SUT->setStarted($this->_id);
        $this->assertTrue($SUT->userHasActiveRestores($this->_unixUser));
    }

    /**
     * test restoresInProgress
     * should return true when restore set to in progress
     * and false otherwise
     */
    public function testRestoresInProgress()
    {
        $SUT = $this->_getSut();

        $SUT->setInProgress($this->_id);
        $this->assertTrue($SUT->restoresInProgress());

        $SUT->setComplete($this->_id);
        $this->assertFalse($SUT->restoresInProgress());

        $SUT->setStarted($this->_id);
        $this->assertTrue($SUT->restoresInProgress());
    }

    /**
     * @return Model_Cache_Restore_Databases
     */
    private function _getSut()
    {
        return $this->_SUT;
    }
}
